#include "NRF24L01.h"

#ifdef SPI_INDEX
#undef SPI_INDEX
#endif

#define SPI_INDEX	NRF_SPI
#include "spi.h"

#include <string.h>
#include <stddef.h>

typedef struct  
{
	unsigned
			incomming_packet: 1,
			pipe_receiving: 1,		// Pipe 0 - default, Pipe 1 - broadcast
			reserved: 6;
	unsigned short rx_timer;
	unsigned short tx_timer;
	unsigned char rx_step;
	unsigned char rx_data_len;
} T_NRF_work;

typedef struct  
{
	uint32_t addr_src;		// Source Address
	uint8_t rq_st;			// Request / Status
} T_NRF_header;

T_NRF_work nrf_work;

#ifndef NRF_STATUS_DISABLED
T_NRF_status nrf_pck_status;
#endif

//-----------------------------------------------------------------------------
void nrf_ms_tick(void)
{
	if (nrf_work.rx_timer)
	{
		nrf_work.rx_timer--;
		if (!nrf_work.rx_timer)
		{
			nrf_work.incomming_packet = false;
			nrf_work.rx_step = 0;
		}			
	}		
	
	if (nrf_work.tx_timer)
		nrf_work.tx_timer--;
}

//-----------------------------------------------------------------------------
uint8_t nrf_r_register(uint8_t address, uint8_t *ret_data, uint8_t data_len)
{
	NRF_SELECT();
	uint8_t nrf_status;
	uint8_t i = 0;
	
	nrf_status = spi_read_write(R_REGISTER | (address & REG_ADDR_MASK));
	
	while (i < data_len)
	{
		ret_data[i] = spi_read();
		i++;
	}
	NRF_UNSELECT();
	return nrf_status;
}

//-----------------------------------------------------------------------------
uint8_t nrf_w_register(uint8_t address, uint8_t *data, uint8_t data_len)
{
	uint8_t nrf_status;
	uint8_t i = 0;
	
	NRF_SELECT();
	
	nrf_status = spi_read_write(W_REGISTER | (address & REG_ADDR_MASK));
	
	while (i < data_len)
	{
		spi_write(data[i]);
		i++;
	}
	NRF_UNSELECT();
	return nrf_status;	
}

//-----------------------------------------------------------------------------
unsigned char nrf_get_status(void)
{
	unsigned char nrf_status;
	NRF_SELECT();
	nrf_status = spi_read_write(NRF_NOP);		// Read status register
	NRF_UNSELECT();
	return nrf_status;
}

//-----------------------------------------------------------------------------
uint8_t nrf_w_register_byte(uint8_t address, uint8_t data)
{
	NRF_SELECT();
	uint8_t nrf_status;
		
	nrf_status = spi_read_write(W_REGISTER | (address & REG_ADDR_MASK));
	spi_write(data);
	
	NRF_UNSELECT();
	return nrf_status;
}

//-----------------------------------------------------------------------------
void nrf_change_mode(e_nrf_mode nrf_mode)		// 1 - receiver, 0 - transmitter
{
	uint8_t data;
	nrf_r_register(0, &data, 1);
	switch (nrf_mode)
	{
		case NRF_PRX :
			NRF_DISABLED();
			nrf_w_register_byte(NRF_CONFIG, (data & 0b01111100) | 0b00000011);
			NRF_ENABLED();
			break;
		
		case NRF_PTX :
			NRF_DISABLED();
			nrf_w_register_byte(NRF_CONFIG, (data & 0b01111100) | 0b00000010);
			break;
		
		case NRF_SLEEP :
			NRF_DISABLED();
			nrf_w_register_byte(NRF_CONFIG, (data & 0b01111100) | 0b00000001);
			break;
			
		default :
			break;
	}
}

//-----------------------------------------------------------------------------
unsigned char nrf_get_payload_width(void)
{
	unsigned char data;
	NRF_SELECT();
	spi_write(R_RX_PL_WID);
	data = spi_read();
	NRF_UNSELECT();
	return data;
}

//-----------------------------------------------------------------------------
void nrf_w_payload(unsigned char * ptr, unsigned char data_len, bool ack_dis)
{
	unsigned char i;
	NRF_SELECT();
	if (ack_dis)
		spi_write(W_TX_PAYLOAD_NO_ACK);	
	else
		spi_write(W_TX_PAYLOAD);
	for (i = 0; i < data_len; i++)
	{
		spi_write(*(ptr + i));
	}
	NRF_UNSELECT();
}

//-----------------------------------------------------------------------------
void nrf_r_payload(unsigned char * ptr, unsigned char lenght)		// return lenght
{
	NRF_SELECT();
	spi_write(R_RX_PAYLOAD);		// Read payload req
	
	if (ptr == NULL)		// Discard payload
	{
		while (lenght > 0)
		{
			spi_read();
			lenght--;
		}
	}
	else					// Receive payload
	{
		while (lenght > 0)
		{
			*ptr = spi_read();
			ptr++;
			lenght--;
		}
	}
	NRF_UNSELECT();	
}

//-----------------------------------------------------------------------------
void nrf_flush_tx(void)
{
	NRF_SELECT();
	spi_write(FLUSH_TX);		// Send first
	NRF_UNSELECT();
}

//-----------------------------------------------------------------------------
void nrf_flush_rx(void)
{
	NRF_SELECT();
	spi_write(FLUSH_RX);		// Send first
	NRF_UNSELECT();
}
//-----------------------------------------------------------------------------
void nrf_activate_features(void)
{
	NRF_SELECT();
	spi_write(ACTIVATE);
	spi_write(0x73);
	NRF_UNSELECT();
	
	// Try write and read
}

//-----------------------------------------------------------------------------
char nrf_is_pck_sent(void)
{
	unsigned char nrf_status;
	
	if (NRF_IRQ)
		return -1;									// Interrupt initiated
	nrf_status = nrf_get_status();					// Read status and retransmited packets
	
	// Transmit OK
	if (nrf_status & TX_DS)							
	{
		nrf_w_register_byte(NRF_STATUS, 0b01110000);		// Read & clear status register
		return 0;
	}
	
	// Error max retransmits
	else if(nrf_status & MAX_RT)					
	{
#ifndef NRF_STATUS_DISABLED
		nrf_pck_status.tx_fail_cnt++;
#endif
		nrf_flush_tx();								// Clear TX & RX buffer
		nrf_flush_rx();
		nrf_w_register_byte(NRF_STATUS, 0b01110000);		// Read & clear status register
		return -1;
	}
	
	// Undefined error - Can receive data in TX mode?
	return -1;
}

//-----------------------------------------------------------------------------
char nrf_send(T_NRF_Packet *pck, bool ack_dis)
{
	unsigned char address_buff[3];
	uint32_t address;
	unsigned char i, nrf_status;
	
	if (nrf_work.incomming_packet)
		return -1;
	
	nrf_status = nrf_w_register_byte(0x07, 0b01110000);	// Read & clear status register
	
	if (nrf_status & TX_FULL)				// Check if TX fifo not full
		return -1; 
	
	nrf_r_register(0x0A, address_buff, 3);			// Save device address
	address = 0xAA0000 | pck->addr_dest;
	nrf_w_register(0x0A, (uint8_t *)&address, 3);	// Pipe 0 address same as addressed device
	nrf_w_register(0x10, (uint8_t *)&address, 3);	// Address to TX
	nrf_change_mode(NRF_PTX);						// Change NRF to TX mode

	NRF_ENABLED();	
	nrf_w_payload((unsigned char *) pck, offsetof(T_NRF_Packet, buffer), ack_dis);
	if (nrf_is_pck_sent() == (unsigned char) -1)	// Transmit error
	{
		NRF_DISABLED();
		nrf_w_register(0x0A, address_buff, 3);		// Return address back
		nrf_w_register(0x10, address_buff, 3);
		nrf_change_mode(NRF_PRX);					// Change NRF to RX mode
		return -1;
	}
	
	i = pck->data_len;
	while (i > 0)
	{
		if ((i / NRF_MAX_PACKET_LEN) > 0)			// Split buffer to separate packets
		{
			nrf_w_payload((pck->buffer + (pck->data_len - i)), NRF_MAX_PACKET_LEN, ack_dis);
			i -= NRF_MAX_PACKET_LEN;
		}
		else
		{
			nrf_w_payload((pck->buffer + (pck->data_len - i)), i, ack_dis);
			i = 0;
		}
		
		if (nrf_is_pck_sent() == (unsigned char) -1)	// Transmit error
		{
			NRF_DISABLED();
			nrf_w_register(0x0A, address_buff, 3);		// Return address back
			nrf_w_register(0x10, address_buff, 3);
			nrf_change_mode(NRF_PRX);					// Change NRF to RX mode
			return -1;
		}
	}
	
	NRF_DISABLED();
	nrf_w_register(0x0A, address_buff, 3);		// Return address back
	nrf_w_register(0x10, address_buff, 3);
	nrf_change_mode(NRF_PRX);					// Change NRF to RX mode
#ifndef NRF_STATUS_DISABLED
	nrf_pck_status.tx_cnt++;
#endif
	return 0;
}

//-----------------------------------------------------------------------------
char nrf_send_pck(T_NRF_Packet *pck)
{
	return nrf_send(pck, false);
}

//-----------------------------------------------------------------------------
char nrf_send_pck_no_ack(T_NRF_Packet *pck)
{
	return nrf_send(pck, true);
}

//-----------------------------------------------------------------------------
char nrf_pck_detect(T_NRF_Packet *pck, unsigned short my_addr)		// return 1-> pck device received
{
	unsigned char nrf_status, payload_len;
	
	if (NRF_IRQ)		// No IRQ, nothing to do
		return 0;
	
	while (!NRF_IRQ)	// While IRQ is low
	{
		nrf_status = nrf_get_status();
		if (nrf_status & RX_DR)
		{
			payload_len = nrf_get_payload_width();
			if (nrf_work.incomming_packet && nrf_work.pipe_receiving != ((nrf_status & RX_P_NO) >> 1))	// discard packet if received from another pipe
			{
				nrf_r_payload(NULL, payload_len);
			}
			else
			{
				if (nrf_work.incomming_packet == 0)		// Checking for start packet
				{
					if (payload_len != offsetof(T_NRF_Packet, buffer))
					{
						nrf_r_payload(NULL, payload_len);	// Discard packet
						continue;
					}							
						
					nrf_r_payload((unsigned char *) pck, payload_len);
					if (pck->addr_dest == my_addr || pck->addr_dest == 0)	// Checking pck lenght & address
					{
						nrf_work.incomming_packet = 1;
						nrf_work.rx_data_len = 0;
						nrf_work.pipe_receiving = ((nrf_status & RX_P_NO) >> 1);
						nrf_work.rx_timer = NRF_RX_TIMEOUT;
					}
				}
				else		// Receiving data in packet
				{
					if (payload_len + nrf_work.rx_data_len <= pck->data_len)
					{
						nrf_r_payload((unsigned char *) pck->buffer + nrf_work.rx_data_len, payload_len);
						nrf_work.rx_data_len += payload_len;
						if (nrf_work.rx_data_len == pck->data_len)		// PCK received
						{
							nrf_work.incomming_packet = 0;
							nrf_work.rx_timer = 0;
							nrf_work.pipe_receiving = 0;
							nrf_work.rx_data_len = 0;
							nrf_w_register_byte(0x07, 0b00110000);		// Read & clear status register unless Data reg.
							nrf_pck_status.rx_cnt++;
							return 1;
						}								
						nrf_work.rx_timer = NRF_RX_TIMEOUT;
					}
					else
					{
						nrf_r_payload(NULL, payload_len);	// Discard packet
						nrf_work.incomming_packet = 0;
						nrf_work.rx_timer = 0;
						nrf_work.pipe_receiving = 0;
						nrf_work.rx_data_len = 0;
					}
				}
			}
		}
	}
	return 0;		
}

//-----------------------------------------------------------------------------
void nrf_init(unsigned long int device_address, unsigned char freq_channel)
{
	uint32_t address;
	
	spi_init();
	
	NRF_DISABLED();										// Shut down NRF module
	NRF_UNSELECT();

	nrf_w_register_byte(NRF_CONFIG, 0b00001011);		// No IRQ masked, EN_CRC, CRC 1byte, pwr_up, PRX
	nrf_w_register_byte(NRF_EN_AA, 0b00000011);			// Auto ack enabled on pipe 0, 1
	nrf_w_register_byte(NRF_EN_RXADDR, 0b00000011);		// Enable datapipes: 0, 1
	nrf_w_register_byte(NRF_SETUP_AW, 0b00000010);		// 4 address bytes
	nrf_w_register_byte(NRF_SETUP_RETR, (1 << 4) | 3);		// Auto retransmit dly: 500us, 3x Auto retransmit
	nrf_w_register_byte(NRF_RF_CH, freq_channel & 0x7F);	// Freqency channel 2,4 GHz + freq_channel in 1 MHz resolution
	nrf_w_register_byte(NRF_RF_SETUP, 0b00000111);		// 1Mbps data rate, Out PWR 0dBm, LNA_gain hi current
	nrf_w_register_byte(NRF_STATUS, 0b01110000);		// Clear status flags
	nrf_w_register_byte(NRF_RX_PW_P0, 32);				// Size of input buffer pipe 0
	nrf_w_register_byte(NRF_RX_PW_P1, 32);				// Size of input buffer pipe 1
	
	address = device_address;
	nrf_w_register(NRF_RX_ADDR_P0, (uint8_t *)&address, 4);		// Pipe 0 address
	nrf_w_register(NRF_TX_ADDR, (uint8_t *)&address, 4);		// Address to TX
	
	address = 0xFFFFFFFF;
	nrf_w_register(NRF_RX_ADDR_P1, (uint8_t *)&address, 4);	// Broadcast address on pipe 1
	
	nrf_w_register_byte(NRF_FEATURE, 0b00000101);		// Enable dynamic payload, enable TX payload no ack
	nrf_activate_features();
	nrf_w_register_byte(NRF_DYNPD, 0b000000011);		// Dynamic payload on datapipes 0, 1
	
	nrf_flush_rx();
	nrf_flush_tx();
}
