#ifndef SOURCES_BME280_H_
#define SOURCES_BME280_H_

#ifndef BMP280_ADDR
#define BMP280_ADDR			0b1110110
#endif

#ifndef BMP280_I2C_SPEED
#define BMP280_I2C_SPEED	400		// kbit/s
#endif

#define BMP_CAL_START_ADDR	0x88
#define BMP_CAL_END_ADDR	0xA1

#define BMP_CAL2_START_ADDR	0xE1
#define BMP_CAL2_END_ADDR	0xE7


enum e_bmp_register
{
	BMP_hum_lsb = 0xFE,
	BMP_hum_msb = 0xFD,
	BMP_temp_xlsb = 0xFC,
	BMP_temp_lsb = 0xFB,
	BMP_temp_msb = 0xFA,
	BMP_press_xlsb = 0xF9,
	BMP_press_lsb = 0xF8,
	BMP_press_msb = 0xF7,
	BMP_config = 0xF5,
	BMP_ctrl_meas = 0xF4,
	BMP_status = 0xF3,
	BMP_ctrl_hum = 0xF2,
	BMP_reset = 0xE0,			// For full reset write 0xB6 to this register
	BMP_id = 0xD0,				// Reading this return always 0x60
	BMP_calib00 = 0x88,
	BMP_calib25 = 0xA1
};

typedef struct
{
	uint16_t dig_T1;
	int16_t dig_T2;
	int16_t dig_T3;
	uint16_t dig_P1;
	int16_t dig_P2;
	int16_t dig_P3;
	int16_t dig_P4;
	int16_t dig_P5;
	int16_t dig_P6;
	int16_t dig_P7;
	int16_t dig_P8;
	int16_t dig_P9;
	uint8_t resvd;
	uint8_t dig_H1;
} BMP_calib_struct;

typedef struct
{
	int16_t dig_H2;
	uint8_t dig_H3;
	int16_t dig_H4;
	int16_t dig_H5;
	uint8_t dig_H6;
} BMP_calib2_struct;

extern uint8_t bmp_init(void);
extern uint8_t bmp_start_measure(void);
extern uint8_t bmp_read_data(int16_t * temp, uint16_t * humid, uint32_t * press);



#endif /* SOURCES_BME280_H_ */
