#include <stdio.h>

#include "fsl_device_registers.h"
#include "main.h"
#include "adc_meas.h"
#include "powermodes.h"
#include "ui.h"
#include "bme280.h"
#include "NRF24L01.h"

#include <spi.h>
#include <systick_wrapper.h>
#include <fsl_clock_manager.h>
#include <fsl_lptmr_driver.h>
#include <fsl_i2c_master_driver.h>
#include <fsl_power_manager.h>
#include <fsl_pit_driver.h>
#include <fsl_pmc_hal.h>
#include <fsl_rtc_driver.h>

// --------------------------------------------------------------------------
/* global variables */
Flag_struct flags;
Timer_struct tmr;
LEDs_struct led;
TState st;

lptmr_state_t lptmr_st;

volatile TMeasureRec meas_history[MEASURE_HIST_DEPTH];

// --------------------------------------------------------------------------
void SysTick_Handler(void)
{
	tmr.timer32++;
	//handle_led(led.error, LedR);
	//handle_led(led.ready, LedB);
	ui_ms_tick();
}

// --------------------------------------------------------------------------
void I2C0_IRQHandler(void)
{
	I2C_DRV_MasterIRQHandler(0);
}

// --------------------------------------------------------------------------
void PORTC_PORTD_IRQHandler(void)
{
	ui_button_scan();

	PORT_HAL_ClearPinIntFlag(PORTC, 1);
	PORT_HAL_ClearPinIntFlag(PORTC, 3);
	PORT_HAL_ClearPinIntFlag(PORTC, 4);
	PORT_HAL_ClearPinIntFlag(PORTD, 4);

	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin5);
	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin6);
	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin7);
	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin8);
	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin14);
}

// --------------------------------------------------------------------------
// RTC alarm IRQ
void RTC_IRQHandler(void)
{
	RTC_HAL_SetAlarmReg(RTC, RTC_HAL_GetSecsReg(RTC) + MEASURE_INTERVAL);
	flags.measure = true;
}

// --------------------------------------------------------------------------
// RTC second IRQ
void RTC_Seconds_IRQHandler(void)
{
	flags.sec = true;
}

// --------------------------------------------------------------------------
// RTC second wakeup enable
void second_wake_enable(void)
{
	flags.sec_wake = true;
}

// --------------------------------------------------------------------------
// Convert time in second to seconds / minutes / hours
void sec_to_time(RTC_struct *rtc, uint32_t seconds)
{
	rtc->hours = seconds / 3600;
	rtc->minutes = (seconds % 3600) / 60;
	rtc->seconds = seconds % 60;
}

// --------------------------------------------------------------------------
unsigned long read_system_timer(void)
{
	unsigned int value;
	value = tmr.timer32;
	return value;
}

// --------------------------------------------------------------------------
void measure_update(void)
{
	static uint8_t avg_cnt = 0;
	static uint32_t avg_hum_in = 0;
	static uint32_t avg_press_in = 0;
	static int32_t avg_temp_in = 0;

	static uint32_t avg_hum_out = 0;
	static uint32_t avg_press_out = 0;
	static int32_t avg_temp_out = 0;

	// Update maximum and minimum temperatures
	if (st.humidity_in_min > st.humidity_in)
		st.humidity_in_min = st.humidity_in;

	if (st.temperature_in_min > st.temperature_in)
		st.temperature_in_min = st.temperature_in;

	if (st.pressure_in_min > st.pressure_in)
		st.pressure_in_min = st.pressure_in;

	if (st.humidity_in_max < st.humidity_in)
		st.humidity_in_max = st.humidity_in;

	if (st.temperature_in_max < st.temperature_in)
		st.temperature_in_max = st.temperature_in;

	if (st.pressure_in_max < st.pressure_in)
		st.pressure_in_max = st.pressure_in;

	// Averaging temperatures
	avg_cnt++;
	avg_hum_in += st.humidity_in;
	avg_hum_out += st.humidity_out;
	avg_temp_in += st.temperature_in;
	avg_temp_out += st.temperature_out;
	avg_press_in += st.pressure_in;
	avg_press_out += st.pressure_out;

	if (avg_cnt < MEASURE_HIST_AVG_SAMP)
	{
		// Store temperatures for graphical representation
		// Move records in memory
		for (uint8_t i = 0; i < (MEASURE_HIST_DEPTH - 1); i++)
		{
			meas_history[(MEASURE_HIST_DEPTH - 1) - i].humidity_in = meas_history[(MEASURE_HIST_DEPTH - 2) - i].humidity_in;
			meas_history[(MEASURE_HIST_DEPTH - 1) - i].humidity_out = meas_history[(MEASURE_HIST_DEPTH - 2) - i].humidity_out;
			meas_history[(MEASURE_HIST_DEPTH - 1) - i].pressure_in = meas_history[(MEASURE_HIST_DEPTH - 2) - i].pressure_in;
			meas_history[(MEASURE_HIST_DEPTH - 1) - i].pressure_out = meas_history[(MEASURE_HIST_DEPTH - 2) - i].pressure_out;
			meas_history[(MEASURE_HIST_DEPTH - 1) - i].temperature_in = meas_history[(MEASURE_HIST_DEPTH - 2) - i].temperature_in;
			meas_history[(MEASURE_HIST_DEPTH - 1) - i].temperature_out = meas_history[(MEASURE_HIST_DEPTH - 2) - i].temperature_out;
		}

		// Add new record
		meas_history[0].humidity_in = avg_hum_in / avg_cnt;
		meas_history[0].humidity_out = avg_hum_out / avg_cnt;
		meas_history[0].pressure_in = avg_press_in / avg_cnt;
		meas_history[0].pressure_out = avg_press_out / avg_cnt;
		meas_history[0].temperature_in = avg_temp_in / avg_cnt;
		meas_history[0].temperature_out = avg_temp_out / avg_cnt;

		avg_cnt = 0;
		avg_hum_in = 0;
		avg_press_in = 0;
		avg_temp_in = 0;
		avg_hum_out = 0;
		avg_press_out = 0;
		avg_temp_out = 0;
	}
}

// --------------------------------------------------------------------------
void measure_init(void)
{
	st.humidity_in_min = 65535;
	st.temperature_in_min = 32767;
	st.pressure_in_min = 32767;
	st.humidity_in_max = 0;
	st.temperature_in_max = -32768;
	st.pressure_in_max = 0;

	for (uint8_t i = 0; i < MEASURE_HIST_DEPTH; i++)
	{
		meas_history[i].humidity_in = UINT16_MAX;
		meas_history[i].humidity_out = UINT16_MAX;
		meas_history[i].pressure_in = INT32_MAX;
		meas_history[i].pressure_out = INT32_MAX;
		meas_history[i].temperature_in = INT16_MAX;
		meas_history[i].temperature_out = INT16_MAX;
	}
}

// --------------------------------------------------------------------------
void measure_find_max_min(TMeasureRec * max, TMeasureRec * min)
{
	// Init values for searching
	max->humidity_in = 0;
	max->humidity_out = 0;
	max->pressure_in = 0;
	max->pressure_out = 0;
	max->temperature_in = -32768;
	max->temperature_out = -32768;

	min->humidity_in = 10000;
	min->humidity_out = 10000;
	min->pressure_in = 900000;
	min->pressure_out = 900000;
	min->temperature_in = 32766;
	min->temperature_out = 32766;

	for (uint8_t i = 0; i < MEASURE_HIST_DEPTH; i++)
	{
		// Indoor humidity
		if ((meas_history[i].humidity_in > max->humidity_in) && meas_history[i].humidity_in != UINT16_MAX)
			max->humidity_in = meas_history[i].humidity_in;
		if ((meas_history[i].humidity_in < min->humidity_in) && meas_history[i].humidity_in != UINT16_MAX)
			min->humidity_in = meas_history[i].humidity_in;

		// Outdoor humidity
		if ((meas_history[i].humidity_out > max->humidity_out) && meas_history[i].humidity_out != UINT16_MAX)
			max->humidity_out = meas_history[i].humidity_out;
		if ((meas_history[i].humidity_out < min->humidity_out) && meas_history[i].humidity_out != UINT16_MAX)
			min->humidity_out = meas_history[i].humidity_out;

		// Indoor pressure
		if ((meas_history[i].pressure_in > max->pressure_in) && meas_history[i].pressure_in != INT32_MAX)
			max->pressure_in = meas_history[i].pressure_in;
		if ((meas_history[i].pressure_in < min->pressure_in) && meas_history[i].pressure_in != INT32_MAX)
			min->pressure_in = meas_history[i].pressure_in;

		// Outdoor pressure
		if ((meas_history[i].pressure_out > max->pressure_out) && meas_history[i].pressure_out != INT32_MAX)
			max->pressure_out = meas_history[i].pressure_out;
		if ((meas_history[i].pressure_out < min->pressure_out) && meas_history[i].pressure_out != INT32_MAX)
			min->pressure_out = meas_history[i].pressure_out;

		// Indoor temperature
		if ((meas_history[i].temperature_in > max->temperature_in) && meas_history[i].temperature_in != INT16_MAX)
			max->temperature_in = meas_history[i].temperature_in;
		if ((meas_history[i].temperature_in < min->temperature_in) && meas_history[i].temperature_in != INT16_MAX)
			min->temperature_in = meas_history[i].temperature_in;

		// Outdoor temperature
		if ((meas_history[i].temperature_out > max->temperature_out) && meas_history[i].temperature_out != INT16_MAX)
			max->temperature_out = meas_history[i].temperature_out;
		if ((meas_history[i].temperature_out < min->temperature_out) && meas_history[i].temperature_out != INT16_MAX)
			min->temperature_out = meas_history[i].temperature_out;
	}
}

// --------------------------------------------------------------------------
void init_sleep(void)
{
	const lptmr_user_config_t cfg = {
			kLptmrTimerModeTimeCounter,
			kLptmrPinSelectInput1,
			kLptmrPinPolarityActiveHigh,
			false,
			true,
			kClockLptmrSrcLpoClk,
			kLptmrPrescalerDivide2,
			false
	};
	__disable_irq();
	LPTMR_DRV_Init(0, &lptmr_st, &cfg);
	LPTMR_DRV_SetTimerPeriodUs(0, WAKE_UP_TIME);
	LPTMR_DRV_InstallCallback(0, NULL);
	__enable_irq();

	// Wakeup on falling edge of pin
	LLWU_HAL_SetExternalInputPinMode(LLWU, kLlwuExternalPinFallingEdge, kLlwuWakeupPin5);		// NRF IRQ
	LLWU_HAL_SetExternalInputPinMode(LLWU, kLlwuExternalPinFallingEdge, kLlwuWakeupPin6);		// Btn down
	LLWU_HAL_SetExternalInputPinMode(LLWU, kLlwuExternalPinFallingEdge, kLlwuWakeupPin7);		// Btn ok
	LLWU_HAL_SetExternalInputPinMode(LLWU, kLlwuExternalPinFallingEdge, kLlwuWakeupPin8);		// Btn up
	LLWU_HAL_SetExternalInputPinMode(LLWU, kLlwuExternalPinFallingEdge, kLlwuWakeupPin14);		// E-paper busy

	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin5);
	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin6);
	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin7);
	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin8);
	LLWU_HAL_ClearExternalPinWakeupFlag(LLWU, kLlwuWakeupPin14);

	pwr_init();
}

// --------------------------------------------------------------------------
void switch_clk(enum e_clk_mode mode)
{
	systick_disable();
	if (mode == E_CLK_FAST_MODE)
	{
		CLOCK_HAL_SetOutDiv1(SIM, 1);		// Core clock division 1x -> 24MHz
	}
	else if (mode == E_CLK_SLOW_MODE)
	{
		CLOCK_HAL_SetOutDiv1(SIM, 6);		// Core clock division 6x -> 4MHz
	}
	SystemCoreClockUpdate();
	SysTick_Config(SystemCoreClock/1000);
	systick_enable();
}

// --------------------------------------------------------------------------
// Sleep until something case interrupt
void wait_to_interrupt(void)
{
	Pindef_DeInit();
	POWER_SYS_SetMode(1U, kPowerManagerPolicyAgreement);
	Pindef_Init();
}

// --------------------------------------------------------------------------
// What to do before sleep
void before_sleep(void)
{
	led.error = 0;
	led.ready = 0;
	LedB_Clear();
	LedR_Clear();
	ui_button_tmr_clear();

	Pindef_DeInit();
	LPTMR_DRV_Stop(0);		// Disable Low power timer (Used for I2C)

	if (flags.sec_wake)		// Disable second wakeup especially if no timer running
		RTC_HAL_SetSecsIntCmd(RTC, true);
	else
		RTC_HAL_SetSecsIntCmd(RTC, false);
}

// --------------------------------------------------------------------------
// What to do after sleep
void after_sleep(void)
{
	PMC_HAL_ClearAckIsolation(PMC);
	Pindef_Init();

	LPTMR_DRV_Start(0);
	RTC_HAL_SetSecsIntCmd(RTC, true);
	flags.sec_wake = false;
}

// --------------------------------------------------------------------------
// Initialize RTC oscillator
void rtc_init(void)
{
	rtc_osc_user_config_t cfg = {
		32768,          /*!< External clock frequency.                 */
		false,    		/*!< Enable 2 pF capacitor load.                */
		false,    		/*!< Enable 4 pF capacitor load.                */
		false,    		/*!< Enable 8 pF capacitor load.                */
		false,   		/*!< Enable 16 pF capacitor load.               */
		true            /*!< Enable OSC or use external clock directly.*/
	};

	CLOCK_SYS_RtcOscInit(0, &cfg);
	RTC_HAL_Init(RTC);
	RTC_HAL_SetSecsIntCmd(RTC, true);
	RTC_HAL_SetAlarmIntCmd(RTC, true);
	RTC_HAL_SetAlarmReg(RTC, RTC_HAL_GetSecsReg(RTC) + 10);
	RTC_HAL_EnableCounter(RTC, true);
	NVIC_EnableIRQ(RTC_Seconds_IRQn);
	NVIC_EnableIRQ(RTC_IRQn);
}

// --------------------------------------------------------------------------
// Execute each second. (only in run mode, not sleep)
void sec_timers_handle(void)
{
	ui_sec_tick();					// UI second tick
}

// --------------------------------------------------------------------------
// Handle Temperature, pressure, humidity measurement
void bmp_measure_handle(void)
{
	uint32_t press;
	static uint8_t read = 0;
	static uint8_t num_updates = 0;

	if (flags.measure == 1)		// Start measuring
	{
		flags.measure = false;
		bmp_start_measure();
		read = 1;
		second_wake_enable();
	}
	else if (read == 1)
	{
		if (!bmp_read_data(&st.temperature_in, &st.humidity_in, &press))
			return;

		read = 0;
		st.pressure_in = press;
		ui.screen_update = true;
		num_updates++;
		if (num_updates > 5)
		{
			ui.partial_update = false;
			num_updates = 0;
		}
		else
			ui.partial_update = true;

		second_wake_enable();
		measure_update();
	}
}

// --------------------------------------------------------------------------
// External flash power down
void extflash_powerdown(void)
{
	EXTFLASH_UNSELECT();
	EXTFLASH_SELECT();
	spi_write(0xB9);
	EXTFLASH_UNSELECT();
}

// --------------------------------------------------------------------------
int main(void)
{
	Pindef_Init();

	SystemCoreClockUpdate();

	init_sleep();

	switch_clk(E_CLK_SLOW_MODE);

	rtc_init();

	nrf_init(DEVICE_ADDRESS, 0);
	nrf_change_mode(NRF_SLEEP);

	spi_init();
	adc_init();

	extflash_powerdown();

	if (adc_check_battery() == 0)		// Battery voltage is very low for driving E-Paper
	{
		second_wake_enable();
		before_sleep();
		while (1)
		{
			LedR_Toggle();				// Blink with error LED
			POWER_SYS_SetMode(1U, kPowerManagerPolicyAgreement);
		}
	}

	bmp_init();
	measure_init();

	ui_init();
	flags.measure = true;

	while(1)
	{
		if (!flags.measure)
		{
			before_sleep();			// Prepare MCU to sleep
			POWER_SYS_SetMode(1U, kPowerManagerPolicyAgreement);
			after_sleep();			// Prepare MCU to run
		}

		adc_handle();
		ui_handle();

		if (flags.sec)
		{
			flags.sec = false;
			st.up_time++;

			st.rtc_time = RTC_HAL_GetSecsReg(RTC);		// Update RTC time
			sec_timers_handle();
			bmp_measure_handle();						// Measuring temperature, humidity and pressure
		}
	}

    return 0;
}

