#ifndef _MAIN_H
#define _MAIN_H

#include "config.h"
#include <Smart_Thermometer.h>

// --------------------------------------------------------------------------
#define handle_led(var, led)	\
{								\
	if (var)					\
	{							\
		if (var != 0xffff)		\
			var--;				\
		LED_PIN(led, 1);		\
	}							\
	else						\
		LED_PIN(led, 0);		\
}

// --------------------------------------------------------------------------
#define handle_timer(timer)			\
{									\
	if (timer)						\
		timer--;					\
}

// --------------------------------------------------------------------------
#define handle_timer_flags(tmr, val, flag)	\
{										\
	if (tmr)							\
		tmr--;							\
	else {								\
		tmr = val;						\
		flag = true;					\
	}									\
}

// --------------------------------------------------------------------------
#define ADC_CHANGE_NAMUR()				{tmr.nam_change = CHANGE_NAM_TIME_FAST_B; }

// --------------------------------------------------------------------------
enum e_clk_mode {
	E_CLK_SLOW_MODE,
	E_CLK_FAST_MODE,
};

// --------------------------------------------------------------------------
typedef struct {
	volatile bool sec;						// second flag
	volatile bool measure;					// perform measuring flag
	volatile bool sec_wake;					// enable second wakeup (if some of timers is running)
} Flag_struct;

typedef struct {
	volatile uint32_t timer32;			// system timer
	volatile uint16_t sec;				// seconds timer
	volatile uint16_t wake_up;			// timer for wakeup
} Timer_struct;

typedef struct {
	volatile uint16_t error;			// Error RED LED
	volatile uint16_t ready;			// Ready Blue LED
} LEDs_struct;

typedef struct
{
	volatile uint8_t hours;
	volatile uint8_t minutes;
	volatile uint8_t seconds;
} RTC_struct;

// --------------------------------------------------------------------------
extern unsigned char outbuff[];
extern volatile TMeasureRec meas_history[MEASURE_HIST_DEPTH];

// --------------------------------------------------------------------------
extern Flag_struct flags;
extern Timer_struct tmr;
extern TState st;

// --------------------------------------------------------------------------
extern void second_wake_enable(void);
extern unsigned long read_system_timer(void);
extern void switch_clk(enum e_clk_mode mode);
extern void wait_to_interrupt(void);
extern void measure_init(void);
extern void measure_find_max_min(TMeasureRec * max, TMeasureRec * min);
extern void sec_to_time(RTC_struct *rtc, uint32_t seconds);

// --------------------------------------------------------------------------
#ifndef main_printf
#define main_printf(...)
#endif

#endif // _MAIN_H
