#ifndef UI_H_
#define UI_H_

#include <stdint.h>

typedef enum
{
	GUI_MAIN,		// Main page
	GUI_MAIN_MAX,	// Main page, maximum temperatures
	GUI_MAIN_MIN,	// Main page, minimum temperatures
	GUI_MENU,		// Menu page
	GUI_GRAPH_TEMP,		// Graph temperature page
	GUI_GRAPH_HUMID,	// Graph humidity page
	GUI_GRAPH_PRESS,	// Graph pressure page
	GUI_CLR_STATS,	// Clear statistics page (max, min)
	GUI_TEST_PAGE,	// Test page
} e_ui_page;

typedef enum
{
	BTN_UP,
	BTN_DOWN,
	BTN_OK,
	BTN_NONE = 255
} e_ui_button;

typedef struct
{
	unsigned
		screen_update: 1,				// Perform screen update
		partial_update: 1,				// Type of screen update: 0 - Full screen update, 1 - Partial screen update
		reserved: 6;

	e_ui_page page;						// Actual UI page
	e_ui_button btn_pressed;			// Pressed button filtered
	e_ui_button btn;					// Pressed button actual
	uint8_t btn_time;					// Button press time
	uint8_t btn_on_screen;				// timer for Button function show on screen
	uint8_t selected_item;				// selected item in menu
	uint8_t return_timer;				// timer for automatically return to main page
} UI_struct;

typedef struct
{
	char text[8];						// Text of message
	uint8_t timer;						// Timer how long the message be displayed
	e_ui_page page;						// Next page after timer end
	uint8_t selected_item;				// Selected item on next page
} UI_Message_struct;


extern UI_struct ui;
extern void ui_handle(void);
extern void ui_ms_tick(void);
extern void ui_sec_tick(void);
extern void ui_init(void);
extern void ui_button_scan(void);
extern void ui_button_tmr_clear(void);

#endif /* UI_H_ */
