#ifndef __FSL_SPI_H__
#define __FSL_SPI_H__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <fsl_device_registers.h>

// --------------------------------------------------------------------------
// External definitions examples
//	#define SPI_INDEX					1
//	#define SPI_1_FREQ					1000000
//	#define	SPI_1_RXBUFFSIZE			128
//	#define	SPI_1_TXBUFFSIZE			128
//	#define SPI_1_SCK					SCK1_PIO0_7
//	#define SPI_1_MOSI					MISO1_PIO0_8
//	#define SPI_1_MISO					MOSI1_PIO0_9

// --------------------------------------------------------------------------
#ifndef SPI_INDEX
#error "SPI_INDEX has to be defined."
#endif

#if FSL_FEATURE_SOC_SPI_COUNT > SPI_INDEX

#include <fsl_spi_master_driver.h>

// --------------------------------------------------------------------------
// Macros SPI
#define __SPI(index, name)			SPI_##index##_##name
#define __PCSPI(index)				PCSSP##index
#define __LPC_SSP(index)			LPC_SSP##index
#define __SPI_VECT(index, vect)		SSP##index##_##vect
#define __SPI_IRQ(index)			SSP##index##_IRQHandler
#define __SPI_CLKDIV(index)			SSP##index##CLKDIV

#define _SPI(index, name)			__SPI(index, name)
#define _PCSPI(index)				__PCSPI(index)
#define _LPC_SSP(index)				__LPC_SSP(index)
#define _SPI_VECT(index, vect)		__SPI_VECT(index, vect)
#define _SPI_IRQ(index)				__SPI_IRQ(index)
#define _SPI_CLKDIV(index)			__SPI_CLKDIV(index)

#define SPI_FREQ					_SPI(SPI_INDEX, FREQ)
/*
#undef  SPI_SEL_DIR
#undef  SPI_SEL
#define SPI_SEL_DIR					_SPI(SPI_INDEX, SEL_DIR)
#define SPI_SEL						_SPI(SPI_INDEX, SEL)
*/
#undef SPI_RXBUFFSIZE
#undef SPI_TXBUFFSIZE

#define SPI_RXBUFFSIZE				_SPI(SPI_INDEX, RXBUFFSIZE)
#define SPI_TXBUFFSIZE				_SPI(SPI_INDEX, TXBUFFSIZE)

#if !((SPI_RXBUFFSIZE > 0) && (SPI_TXBUFFSIZE > 0)) && ((SPI_RXBUFFSIZE > 0) || (SPI_TXBUFFSIZE > 0))
#error "Both or none of SPI_RXBUFFSIZE and SPI_TXBUFFSIZE have to be nonzero."
#endif

#define SPI_Master_Init				_SPI(SPI_INDEX, Master_Init)
#define SPI_Slave_Init				_SPI(SPI_INDEX, Slave_Init)
#define SPI_Disable					_SPI(SPI_INDEX, Disable)
#define SPI_Read					_SPI(SPI_INDEX, Read)
#define SPI_Write					_SPI(SPI_INDEX, Write)
#define SPI_Read_Write				_SPI(SPI_INDEX, Read_Write)
#define SPI_ms_tick					_SPI(SPI_INDEX, ms_tick)
#define SPI_STATE					_SPI(SPI_INDEX, STATE)
#define spi							_SPI(SPI_INDEX, state)
#define SPI_Write_Fifo				_SPI(SPI_INDEX, Write_Fifo)
#define SPI_Read_Fifo				_SPI(SPI_INDEX, Read_Fifo)
#define SPI_Change_Mode				_SPI(SPI_INDEX, Change_Mode)

// --------------------------------------------------------------------------
// Options SPI
// Frame format: 0x00 - SPI, 0x01 - TI, 0x02 - Microwire
#if defined(SPI_FRAME_FORMAT) && (SPI_FRAME_FORMAT != 0)
#error "Only SPI format supported!"
#endif
// Clock out idle polarity 0 - low 1 - high
#ifndef SPI_COP
#define SPI_COP				0
#endif
// Clock out phase 0 - away from, 1 - back to
#ifndef SPI_COPH
#define SPI_COPH			0
#endif
// Bit order: 0 - MSB first, 1 - LSB first
#ifndef SPI_BIT_ORDER
#define SPI_BIT_ORDER		0
#endif

#define SPI_FIFOSIZE		8

// --------------------------------------------------------------------------
// SPI MODES
#define SPI_MODE_0			0	//	CPOL = 0, CPHA = 0
#define SPI_MODE_1			1	//	CPOL = 0, CPHA = 1
#define SPI_MODE_2			2	//	CPOL = 1, CPHA = 0
#define SPI_MODE_3			3	//	CPOL = 1, CPHA = 1

// --------------------------------------------------------------------------
//Define struct
typedef struct {
#if SPI_RXBUFFSIZE > 0
	unsigned int rx_incnt;
	unsigned int rx_outcnt;
#if SPI_LENGTH_WORD <= 8
	unsigned char rx_buffer[SPI_RXBUFFSIZE];
#else
	unsigned int rx_buffer[SPI_RXBUFFSIZE];
#endif
#endif

#if SPI_TXBUFFSIZE > 0
	unsigned int tx_incnt;
	unsigned int tx_outcnt;
#if SPI_LENGTH_WORD <= 8
	unsigned char tx_buffer[SPI_TXBUFFSIZE];
#else
	unsigned int tx_buffer[SPI_TXBUFFSIZE];
#endif
#endif

	spi_master_user_config_t config;
	unsigned char rx_tim;
	unsigned char tx_tim;
	unsigned
		rx_overflow: 1,
		tx_empty: 1,
		tx_overflow: 1;
} SPI_STATE;

// --------------------------------------------------------------------------
// Declaration function
#ifndef spi_init
#define spi_init()			SPI_Master_Init(SPI_FREQ)
#endif
#ifndef spi_write
#define spi_write(data)		SPI_Write(data)
#endif
#ifndef spi_write_fifo
#define spi_write_fifo(data)	SPI_Write_Fifo(data)
#endif
#ifndef spi_read_write
#define spi_read_write(data)	SPI_Read_Write(data)
#endif
#ifndef spi_read
#define spi_read()			SPI_Read_Write(0x00)
#endif
#ifndef spi_read_fifo
#define spi_read_fifo()		SPI_Read_Fifo()
#endif

#ifdef __cplusplus
extern "C" {
#endif

extern void SPI_Master_Init(unsigned int freq);
extern void SPI_Slave_Init(void);
extern void SPI_Disable(void);
extern unsigned int SPI_Read(void);
extern void SPI_Write(unsigned int data);
extern unsigned int SPI_Read_Write(unsigned int data);
extern void SPI_ms_tick();
extern void SPI_Write_Fifo(unsigned int data);
extern unsigned int SPI_Read_Fifo(void);
extern void SPI_Change_Mode(unsigned char mode);
// --------------------------------------------------------------------------
// global variable
extern volatile SPI_STATE spi;
// --------------------------------------------------------------------------

#ifdef __cplusplus
} // extern "C"
#endif

#endif // FSL_FEATURE_SOC_SPI_COUNT > SPI_INDEX
#endif // __FSL_SPI_H__
