#include "systick_wrapper.h"
#include <fsl_interrupt_manager.h>

#if (__Vendor_SysTickConfig == 0)

/********************************************************************
*    Function Name: init_systick		               		        *
*    Return Value: void                                             *
*    Parameters: interval, callback                                 *
*                                                                   *
*    Description: Initialization System tick						*
*    Note: interval [us]										    *
*    										                        *
********************************************************************/
void init_systick (unsigned int interval, void (*callback)(void))
{
	uint64_t calc;

	// Compute required ticks per interval
	SystemCoreClockUpdate();
	calc  = SystemCoreClock;
	calc *= interval;
	calc /= 1000000;

	// Register new callback if required
	if (callback != NULL)
		INT_SYS_InstallHandler(SysTick_IRQn, callback);

	// Configure and enable SysTick
	SysTick_Config((uint32_t)calc);
}

void systick_enable(void)
{
	__disable_irq();
	SysTick->CTRL  |= SysTick_CTRL_ENABLE_Msk;
	__enable_irq();
}

void systick_disable(void)
{
	__disable_irq();
	SysTick->CTRL  &= ~SysTick_CTRL_ENABLE_Msk;
	__enable_irq();
}

#endif // (__Vendor_SysTickConfig == 0)
